"""add secret_type to to secrets table

Revision ID: 2f91aed0b218
Revises: fa67e2abfd00
Create Date: 2024-11-03 23:38:43.477126

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
import sqlmodel  # noqa F401


# revision identifiers, used by Alembic.
revision: str = "2f91aed0b218"
down_revision: Union[str, None] = "fa67e2abfd00"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("secrets", sa.Column("secret_type", sa.TEXT(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("secrets", "secret_type")
    # ### end Alembic commands ###
