from admyral.actions.integrations.vulnerability_management.amazon_inspector2 import (
    list_amazon_inspector2_vulnerabilities,
)
from admyral.actions.integrations.vulnerability_management.snyk import (
    list_snyk_vulnerabilities,
)

__all__ = [
    "list_amazon_inspector2_vulnerabilities",
    "list_snyk_vulnerabilities",
]
