"""change default user email

Revision ID: fa67e2abfd00
Revises: 2c07323c94bf
Create Date: 2024-11-03 16:58:03.800495

"""

from typing import Sequence, Union

from alembic import op
import sqlmodel  # noqa F401


# revision identifiers, used by Alembic.
revision: str = "fa67e2abfd00"
down_revision: Union[str, None] = "2c07323c94bf"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


DEFAULT_USER_ID = "38815447-e272-4299-94c0-29a2d30435f9"


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###

    op.execute(
        f"UPDATE \"User\" SET email = 'default.user@admyral.ai' WHERE id = '{DEFAULT_USER_ID}'",
    )

    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###

    op.execute(
        f"UPDATE \"User\" SET email = 'random@email.com' WHERE id = '{DEFAULT_USER_ID}'",
    )

    # ### end Alembic commands ###
